
from argparse import ArgumentParser


def create_parser():
    parser = ArgumentParser(description="""
    Automate Tokens Generation
    """)
    parser.add_argument(
        "--vault", '-v',
        help='Azure keyvault name',
        nargs=1,
    )
    parser.add_argument(
        "--vault_url", '-vu',
        help='Keyvault URL',
        nargs=1,
    )
    parser.add_argument(
        "--secret", '-s',
        help='Secret name in the keyvault',
        nargs=1,
    )
    parser.add_argument(
        "--retailer", '-r',
        help='Retailer Name',
        nargs=1,
        default='morrisons-favourites',
        required=True
    )
    parser.add_argument(
        "--username", '-',
        help='Username of GitLab Registry',
        nargs=1,
        required=True
    )
    parser.add_argument(
        "--email", '-e',
        help='Email of GitLab Registry',
        nargs=1,
        required=True
    )
    parser.add_argument(
        "--password", '-p',
        help='Password of GitLab Registry',
        nargs=1,
        required=True
    )
   
    return parser


def main():
    from tokengen import source
    import logging
    import os
    import sys


    """Configure logger to stream to stdout"""
    logging.basicConfig(stream=sys.stdout, level=logging.INFO)
    logger = logging.getLogger("tokengen")

    from tokengen import source


    args = create_parser().parse_args()

    
    """Generate the tokens and push to Azure keyvault"""
    source.generate_secret_value_and_push_to_azkeyvault(''.join(args.secret), ''.join(args.vault_url))

    """Use Pepper token and retailer to generate token"""
    source.generate_oasis_api_token(
        username=''.join(args.username),
        email=''.join(args.email),
        password=''.join(args.password), 
        retailer=''.join(args.retailer))

    print("AIFI_API_TOKEN: ", os.getenv("AIFI_API_TOKEN"))
    print("PEPPER: ", os.getenv("PEPPER"))

    print('All Processes completed!')


if __name__ == "__main__":
    import time

    start_time = time.perf_counter()
    main()
    end_time = time.perf_counter()

    timeout = end_time - start_time
    print(f'It took {timeout :0.2f} second(s) to complete.')





from setuptools import find_packages, setup

with open('README.md', 'r') as f:
    long_description = f.read()

with open("requirements.txt", "r", encoding="utf-8") as fh:
    requirements = fh.read()

setup (
    name='tokengen',
    version='0.0.1',
    author='Sharhan Alhassan',
    author_email='sharhan.alhassan@tiacloud.co',
    description='A simple CLI utility for automating token generations with kubernetes.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='',
    packages=find_packages('src'),
    package_dir={'': 'src'},
    install_requires=[requirements],
    python_requires='>=3.9',
    classifiers=[
        "Programming Language :: Python :: 3.9",
        "License :: OSI Approved :: MIT License",
    ],
    entry_points={
        'console_scripts': [
            'tokengen=tokengen.cli:main',
        ],
    }
)



from azure.identity import DefaultAzureCredential
from azure.keyvault.secrets import SecretClient
import secrets
import docker
from docker.errors import APIError, ContainerError
import logging
import sys
import os


"""Configure logger to stream to stdout"""
logging.basicConfig(stream=sys.stdout, level=logging.INFO)
logger = logging.getLogger("tokengen")


credential = DefaultAzureCredential()

def generate_30_hex_secret():
    """hex(16) -- 32 characters"""
    AIFI_API_TOKEN = secrets.token_hex(16)

    return AIFI_API_TOKEN


def generate_80_hex_secret():
    """hex(40) -- 80 characters"""
    PEPPER = secrets.token_hex(40)

    return PEPPER


def generate_secret_value_and_push_to_azkeyvault(secret_name, vault_url):
    """
    Authentication procedure -- Azure CLI login
    Target -- Environment variables
    """
    secret_client = SecretClient(vault_url=vault_url, credential=credential)

    hex_30_secret = generate_30_hex_secret()
    hex_80_secret = generate_80_hex_secret()

    try:
        """Send AIFI_API_TOKEN and PEPPER tokens to the secret in the Azurekeyvault"""
        secret = secret_client.set_secret(
            secret_name, 
            "AIFI_API_TOKEN: {0}\nPEPPER: {1}".format(hex_30_secret, hex_80_secret)
            )

        os.environ['AIFI_API_TOKEN'] = hex_30_secret
        os.environ['PEPPER'] = hex_80_secret

    except Exception as e:
        logger.error(e)

    # return hex_30_secret, hex_80_secret
    # Uncomment if we no longer want to export variables to stdout but return then to another function


def get_secret_value(vault_url, secret_name):

    secret_client = SecretClient(vault_url=vault_url, credential=credential)
    secret = secret_client.get_secret(secret_name)

    logger.info(secret.name)
    logger.info(secret.value)


def generate_oasis_api_token(username=None, password=None, email=None, registry=None, retailer=None):
    client = docker.from_env()
    try:
        client.login(
            username=username, 
            password=password, 
            email=email,
            registry='registry.gitlab.com')

        logger.info("Successfully Logged in to registry!")
    except APIError as e:
        print(e)

    try:
        """ auto_remove=True removes container after it exits """
        image_name = "registry.gitlab.com/aifi-ml/production/cloud-api/utils:2.112.0"
        command_args = "'node' './build/src/scripts/generateAdminToken.js'"
        environment_variables = ["PEPPER={0}".format(os.getenv("PEPPER")), "RETAILER=retailer".format(retailer)]

        container = client.containers.run(image_name, command_args, environment=environment_variables, detach=True)

        for log in container.logs(stream=True):
           logger.info(str(log.strip(), encoding="utf-8"))

    except ContainerError as e:
        logger.error(e)